#!-PERL-
# fm2mif.pl
# Copyright : Norwegian Telecom Research, 1993
# Programming : Jon Stephenson von Tetzchner
# This file can be changed and modified as long as this
# header is kept intact. This code can not be included
# in a commercial product.

# This script is part of the fm2html package.

$, = " ";		# set output field separator
$\ = "\n";		# set output record separator

$FALSE = 0;
$TRUE = 1;

$FM_MAGIC	= "<MakerFile";
$FM_BOOK_MAGIC	= "<Bookfile";
$MIF_MAGIC	= "<MIFFile";
$MIF_BOOK_MAGIC	= "<Book ";

$TMPNAME	= "fm2mif.fmbatch";

local( $tmpdir, $tmpfile, $infile, $outfile, $doc );

$tmpdir = $ENV{'TMPDIR'};
if ( ! $tmpdir ) { $tmpdir = "-TMPDIR-"; }
if ( ! $tmpdir ) { $tmpdir = `pwd`; chop( $tmpdir ); }
$tmpdir =~ s#/$##;
$tmpfile = "$tmpdir/$TMPNAME.$$";

$strip_doc_extension = $FALSE;

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;
  
  if ( m|^-s| )
  {
    $strip_doc_extension = $TRUE; next;
  }

  die "Invalid flag: \"$_\"";
}

if ( $#ARGV != 0 )
{
  print "\n\tUsage: fm2mif [-s] mif-file";
  print "";
  print "\tThe parameter specifies the name of a FrameMaker file to";
  print "\ttranslate to Maker Interchange Format (MIF).  If a FrameMaker";
  print "\tbook file is specified then all files in the book will be";
  print "\ttranslated, as will the book itself.";
  print "";
  print "\tThe name of each output file is formed by appending \".mif\"";
  print "\tto the name of the input file.  If the name of an input file";
  print "\tends in a \".doc\" extension and the \"-s\" (for \"strip\")";
  print "\toption has been specified, the extension will be removed";
  print "\tprior to \".mif\" being appended.";
  print "";
  exit( 1 );
}

$infile = $ARGV[0];

if ( ! -e "$infile" )
  { die "fm2mif: ERROR: Input file \"$infile\" does not exist... exiting"; }
if ( ! open( INFILE, "<$infile" ) )
  { die "fm2mif: ERROR: Could not open input file \"$infile\"... exiting"; }

$idline = <INFILE>;

if ( $idline =~ /^$MIF_MAGIC/i || $idline =~ /^$MIF_BOOK_MAGIC/i )
  { die "fm2mif: ERROR: File \"$infile\" is already in MIF format... exiting"; }
if ( $idline !~ /^$FM_MAGIC/i && $idline !~ /^$FM_BOOK_MAGIC/i )
  { die "fm2mif: ERROR: File \"$infile\" is not a FrameMaker document... exiting"; }

close( INFILE );

if ( $idline =~ /^$FM_BOOK_MAGIC/i )
{
  local( @docs );

  $outfile = $infile;
  if ( $strip_doc_extension == $TRUE )
    { $outfile =~ s/\.doc$//; }
  $outfile = "$outfile.mif";

  print STDERR
    "fm2mif: Converting book file \"$infile\" to MIF file \"$outfile\"...";
  
  open( TMPF, ">$tmpfile" ) ||
    die "fm2mif: ERROR: Could not create temporary file $tmpfile... exiting";
  print TMPF "Open $infile\nSaveAs m $infile $outfile\nQuit";
  close( TMPF );
  system( "-FMBATCH- $tmpfile" );
  system( "rm -f $tmpfile" );

  if ( ! -e "$outfile" )
    {die "fm2mif: ERROR: Failed to convert FrameMaker book to MIF... exiting";}
  if ( ! open( BOOKMIF, "<$outfile" ) )
    { die "fm2mif: ERROR: Could not open MIF file \"$outfile\"... exiting"; }

  while ( <BOOKMIF> )
  {
    if ( /<FileName[^`]*`<c\\>[^<]*<U\\>([^']*)'>/ )
    {
      $docs[++$#docs] = $1; 
    }
    elsif ( /<FileName[^`]*`<c\\>([^']*)/ )
    {
      $docs[++$#docs] = $1; 
    }			    
  }

  close( BOOKMIF );
  @ARGV = @docs;
}

open( TMPF, ">$tmpfile" )
  || die "fm2mif: ERROR: Could not create temporary file $tmpfile... exiting";
print STDERR 
  "fm2mif: Translating the following FrameMaker document files to MIF:";

while ( $doc = shift )
{
  if ( ! -e "$doc" )
  {
    print STDERR "fm2mif: ERROR: Can't convert file \"$doc\" (does not exist)";
    next;
  }

  if ( ! open( DOC, "<$doc" ) )
  {
    print STDERR "fm2mif: ERROR: Can't open file \"$doc\"";
    next;
  }

  $idline = <DOC>;
  close( DOC );

  if ( $idline =~ /^$MIF_MAGIC/i || $idline =~ /^$MIF_BOOK_MAGIC/i )
  {
    print STDERR "fm2mif: ATTENTION: File \"$doc\" is already in MIF format";
    next;
  }

  if ( $idline !~ /^$FM_MAGIC/i && $idline !~ /^$FM_BOOK_MAGIC/i )
  {
    print STDERR "fm2mif: ERROR: File \"$doc\" is not a FrameMaker document";
    next;
  }

  $mif = $doc;
  if ( $strip_doc_extension == $TRUE )
    { $mif =~ s/\.doc$//; }
  $mif = "$mif.mif";
  
  print TMPF "Open $doc\nSaveAs m $doc $mif";
  print STDERR " $doc";
}

print TMPF "Quit"; 
close( TMPF );

system( "-FMBATCH- $tmpfile" );
system( "rm -f $tmpfile" );

print STDERR "fm2mif: Done";

